@@ -1,14 +1,13 @@
 //! Test helpers for ntex http client to use during testing.
 use std::convert::TryFrom;
 
-use bytes::Bytes;
-
 #[cfg(feature = "cookie")]
 use coo_kie::{Cookie, CookieJar};
 
 use crate::http::error::HttpError;
 use crate::http::header::{HeaderName, HeaderValue};
 use crate::http::{h1, Payload, ResponseHead, StatusCode, Version};
+use crate::util::Bytes;
 
 use super::ClientResponse;
 
@@ -97,7 +96,7 @@ impl TestResponse {
                     percent_encode(c.name().as_bytes(), crate::http::helpers::USERINFO);
                 let value =
                     percent_encode(c.value().as_bytes(), crate::http::helpers::USERINFO);
-                let _ = write!(&mut cookie, "; {}={}", name, value);
+                let _ = write!(cookie, "; {}={}", name, value);
             }
             if !cookie.is_empty() {
                 head.headers.insert(
