@@ -7,7 +7,7 @@ use super::Writer;
 use crate::http::error::PayloadError;
 use crate::http::header::{ContentEncoding, HeaderMap, CONTENT_ENCODING};
 use crate::rt::{spawn_blocking, JoinHandle};
-use crate::{util::Bytes, Stream};
+use crate::util::{Bytes, Stream};
 
 const INPLACE: usize = 2049;
 
