@@ -1,19 +1,43 @@
 use std::{
-    cell::RefCell, fmt, future::Future, io, marker::PhantomData, mem, net, pin::Pin,
-    rc::Rc,
+    cell::Cell, cell::RefCell, fmt, future::Future, io, marker::PhantomData, mem, net,
+    pin::Pin, rc::Rc,
 };
 
 use log::error;
 
-use crate::rt::net::TcpStream;
-use crate::service;
-use crate::util::{counter::CounterGuard, HashMap, Ready};
+use crate::util::{HashMap, Ready};
+use crate::{io::Io, service, util::PoolId};
 
-use super::builder::bind_addr;
 use super::service::{
     BoxedServerService, InternalServiceFactory, ServerMessage, StreamService,
 };
 use super::Token;
+use super::{builder::bind_addr, counter::CounterGuard};
+
+#[derive(Clone)]
+pub struct Config(pub(super) Rc<InnerServiceConfig>);
+
+pub(super) struct InnerServiceConfig {
+    pub(super) pool: Cell<PoolId>,
+}
+
+impl Default for Config {
+    fn default() -> Self {
+        Self(Rc::new(InnerServiceConfig {
+            pool: Cell::new(PoolId::DEFAULT),
+        }))
+    }
+}
+
+impl Config {
+    /// Set memory pool for the service.
+    ///
+    /// Use specified memory pool for memory allocations.
+    pub fn memory_pool(&self, id: PoolId) -> &Self {
+        self.0.pool.set(id);
+        self
+    }
+}
 
 pub struct ServiceConfig {
     pub(super) services: Vec<(String, net::TcpListener)>,
@@ -24,7 +48,7 @@ pub struct ServiceConfig {
 }
 
 impl ServiceConfig {
-    pub(super) fn new(threads: usize, backlog: i32) -> ServiceConfig {
+    pub(super) fn new(threads: usize, backlog: i32) -> Self {
         ServiceConfig {
             threads,
             backlog,
@@ -55,11 +79,7 @@ impl ServiceConfig {
     }
 
     /// Add new service to the server.
-    pub fn listen<N: AsRef<str>>(
-        &mut self,
-        name: N,
-        lst: net::TcpListener,
-    ) -> &mut Self {
+    pub fn listen<N: AsRef<str>>(&mut self, name: N, lst: net::TcpListener) -> &mut Self {
         if !self.applied {
             self.apply = Box::new(ConfigWrapper {
                 f: |_| {
@@ -73,40 +93,6 @@ impl ServiceConfig {
         self
     }
 
-    #[doc(hidden)]
-    #[deprecated(since = "0.4.13", note = "Use .on_worker_start() instead")]
-    /// Register service configuration function.
-    ///
-    /// This function get called during worker runtime configuration.
-    /// It get executed in the worker thread.
-    pub fn apply<F>(&mut self, f: F) -> io::Result<()>
-    where
-        F: Fn(&mut ServiceRuntime) + Send + Clone + 'static,
-    {
-        self.on_worker_start::<_, Ready<(), &'static str>, &'static str>(
-            move |mut rt| {
-                f(&mut rt);
-                Ready::Ok(())
-            },
-        )
-    }
-
-    #[doc(hidden)]
-    #[deprecated(since = "0.4.13", note = "Use .on_worker_start() instead")]
-    /// Register async service configuration function.
-    ///
-    /// This function get called during worker runtime configuration.
-    /// It get executed in the worker thread.
-    pub fn apply_async<F, R, E>(&mut self, f: F) -> io::Result<()>
-    where
-        F: Fn(ServiceRuntime) -> R + Send + Clone + 'static,
-        R: Future<Output = Result<(), E>> + 'static,
-        E: fmt::Display + 'static,
-    {
-        self.on_worker_start(f)?;
-        Ok(())
-    }
-
     /// Register async service configuration function.
     ///
     /// This function get called during worker runtime configuration stage.
@@ -163,8 +149,7 @@ impl InternalServiceFactory for ConfiguredService {
 
     fn create(
         &self,
-    ) -> Pin<Box<dyn Future<Output = Result<Vec<(Token, BoxedServerService)>, ()>>>>
-    {
+    ) -> Pin<Box<dyn Future<Output = Result<Vec<(Token, BoxedServerService)>, ()>>>> {
         // configure services
         let rt = ServiceRuntime::new(self.topics.clone());
         let cfg_fut = self.rt.configure(ServiceRuntime(rt.0.clone()));
@@ -198,12 +183,13 @@ impl InternalServiceFactory for ConfiguredService {
                     let name = names.remove(&token).unwrap().0;
                     res.push((
                         token,
-                        Box::new(StreamService::new(service::fn_service(
-                            move |_: TcpStream| {
+                        Box::new(StreamService::new(
+                            service::fn_service(move |_: Io| {
                                 error!("Service {:?} is not configured", name);
                                 Ready::<_, ()>::Ok(())
-                            },
-                        ))),
+                            }),
+                            PoolId::P0,
+                        )),
                     ));
                 };
             }
@@ -291,8 +277,23 @@ impl ServiceRuntime {
     /// *ServiceConfig::bind()* or *ServiceConfig::listen()* methods.
     pub fn service<T, F>(&self, name: &str, service: F)
     where
-        F: service::IntoServiceFactory<T>,
-        T: service::ServiceFactory<Config = (), Request = TcpStream> + 'static,
+        F: service::IntoServiceFactory<T, Io>,
+        T: service::ServiceFactory<Io> + 'static,
+        T::Future: 'static,
+        T::Service: 'static,
+        T::InitError: fmt::Debug,
+    {
+        self.service_in(name, PoolId::P0, service)
+    }
+
+    /// Register service with memory pool.
+    ///
+    /// Name of the service must be registered during configuration stage with
+    /// *ServiceConfig::bind()* or *ServiceConfig::listen()* methods.
+    pub fn service_in<T, F>(&self, name: &str, pool: PoolId, service: F)
+    where
+        F: service::IntoServiceFactory<T, Io>,
+        T: service::ServiceFactory<Io> + 'static,
         T::Future: 'static,
         T::Service: 'static,
         T::InitError: fmt::Debug,
@@ -303,6 +304,7 @@ impl ServiceRuntime {
             inner.services.insert(
                 token,
                 Box::new(ServiceFactory {
+                    pool,
                     inner: service.into_factory(),
                 }),
             );
@@ -322,11 +324,10 @@ impl ServiceRuntime {
 
 type BoxedNewService = Box<
     dyn service::ServiceFactory<
-        Request = (Option<CounterGuard>, ServerMessage),
+        (Option<CounterGuard>, ServerMessage),
         Response = (),
         Error = (),
         InitError = (),
-        Config = (),
         Service = BoxedServerService,
         Future = Pin<Box<dyn Future<Output = Result<BoxedServerService, ()>>>>,
     >,
@@ -334,29 +335,29 @@ type BoxedNewService = Box<
 
 struct ServiceFactory<T> {
     inner: T,
+    pool: PoolId,
 }
 
-impl<T> service::ServiceFactory for ServiceFactory<T>
+impl<T> service::ServiceFactory<(Option<CounterGuard>, ServerMessage)> for ServiceFactory<T>
 where
-    T: service::ServiceFactory<Config = (), Request = TcpStream>,
+    T: service::ServiceFactory<Io>,
     T::Future: 'static,
     T::Service: 'static,
     T::Error: 'static,
     T::InitError: fmt::Debug + 'static,
 {
-    type Request = (Option<CounterGuard>, ServerMessage);
     type Response = ();
     type Error = ();
     type InitError = ();
-    type Config = ();
     type Service = BoxedServerService;
     type Future = Pin<Box<dyn Future<Output = Result<BoxedServerService, ()>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
+        let pool = self.pool;
         let fut = self.inner.new_service(());
         Box::pin(async move {
             match fut.await {
-                Ok(s) => Ok(Box::new(StreamService::new(s)) as BoxedServerService),
+                Ok(s) => Ok(Box::new(StreamService::new(s, pool)) as BoxedServerService),
                 Err(e) => {
                     error!("Cannot construct service: {:?}", e);
                     Err(())
