@@ -1,19 +1,15 @@
 use std::{fmt, io, marker::PhantomData, net, sync::Arc, sync::Mutex};
 
 #[cfg(feature = "openssl")]
-use crate::server::openssl::{AlpnError, SslAcceptor, SslAcceptorBuilder};
+use tls_openssl::ssl::{AlpnError, SslAcceptor, SslAcceptorBuilder};
 #[cfg(feature = "rustls")]
-use crate::server::rustls::ServerConfig as RustlsServerConfig;
+use tls_rustls::ServerConfig as RustlsServerConfig;
 
-#[cfg(unix)]
-use crate::http::Protocol;
 use crate::http::{
     body::MessageBody, HttpService, KeepAlive, Request, Response, ResponseError,
 };
 use crate::server::{Server, ServerBuilder};
-#[cfg(unix)]
-use crate::service::pipeline_factory;
-use crate::{service::map_config, IntoServiceFactory, Service, ServiceFactory};
+use crate::service::{map_config, IntoServiceFactory, ServiceFactory};
 use crate::{time::Seconds, util::PoolId};
 
 use super::config::AppConfig;
@@ -47,11 +43,10 @@ struct Config {
 pub struct HttpServer<F, I, S, B>
 where
     F: Fn() -> I + Send + Clone + 'static,
-    I: IntoServiceFactory<S>,
-    S: ServiceFactory<Config = AppConfig, Request = Request>,
+    I: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig>,
     S::Error: ResponseError,
     S::InitError: fmt::Debug,
-    <S::Service as Service>::Future: 'static,
     S::Response: Into<Response<B>>,
     B: MessageBody,
 {
@@ -65,13 +60,11 @@ where
 impl<F, I, S, B> HttpServer<F, I, S, B>
 where
     F: Fn() -> I + Send + Clone + 'static,
-    I: IntoServiceFactory<S>,
-    S: ServiceFactory<Config = AppConfig, Request = Request>,
-    S::Error: ResponseError + 'static,
+    I: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig> + 'static,
+    S::Error: ResponseError,
     S::InitError: fmt::Debug,
-    S::Future: 'static,
-    S::Response: Into<Response<B>> + 'static,
-    <S::Service as Service>::Future: 'static,
+    S::Response: Into<Response<B>>,
     B: MessageBody + 'static,
 {
     /// Create new http server with application factory
@@ -84,7 +77,7 @@ where
                 client_timeout: Seconds(5),
                 client_disconnect: Seconds(5),
                 handshake_timeout: Seconds(5),
-                pool: PoolId::P1,
+                pool: PoolId::P0,
             })),
             backlog: 1024,
             builder: ServerBuilder::default(),
@@ -135,7 +128,7 @@ where
     ///
     /// By default max connections is set to a 256.
     pub fn maxconnrate(self, num: usize) -> Self {
-        crate::server::max_concurrent_ssl_accept(num);
+        ntex_tls::max_concurrent_ssl_accept(num);
         self
     }
 
@@ -227,28 +220,12 @@ where
 
     /// Set memory pool.
     ///
-    /// Use specified memory pool for memory allocations. By default P1
-    /// memory pool is used.
-    pub fn memory_pool(self, id: PoolId) -> Self {
+    /// Use specified memory pool for memory allocations.
+    pub fn memory_pool(&mut self, id: PoolId) -> &mut Self {
         self.config.lock().unwrap().pool = id;
         self
     }
 
-    #[doc(hidden)]
-    #[deprecated(since = "0.4.12", note = "Use memory pool config")]
-    #[inline]
-    /// Set read/write buffer params
-    ///
-    /// By default read buffer is 8kb, write buffer is 8kb
-    pub fn buffer_params(
-        self,
-        _max_read_buf_size: u16,
-        _max_write_buf_size: u16,
-        _min_buf_size: u16,
-    ) -> Self {
-        self
-    }
-
     /// Use listener for accepting incoming connection requests
     ///
     /// HttpServer does not change any configuration for TcpListener,
@@ -258,26 +235,23 @@ where
         let factory = self.factory.clone();
         let addr = lst.local_addr().unwrap();
 
-        self.builder = self.builder.listen(
-            format!("ntex-web-service-{}", addr),
-            lst,
-            move || {
-                let c = cfg.lock().unwrap();
-                let cfg = AppConfig::new(
-                    false,
-                    addr,
-                    c.host.clone().unwrap_or_else(|| format!("{}", addr)),
-                );
+        self.builder =
+            self.builder
+                .listen(format!("ntex-web-service-{}", addr), lst, move |r| {
+                    let c = cfg.lock().unwrap();
+                    let cfg = AppConfig::new(
+                        false,
+                        addr,
+                        c.host.clone().unwrap_or_else(|| format!("{}", addr)),
+                    );
+                    r.memory_pool(c.pool);
 
-                HttpService::build()
-                    .keep_alive(c.keep_alive)
-                    .client_timeout(c.client_timeout)
-                    .disconnect_timeout(c.client_disconnect)
-                    .memory_pool(c.pool)
-                    .finish(map_config(factory(), move |_| cfg.clone()))
-                    .tcp()
-            },
-        )?;
+                    HttpService::build()
+                        .keep_alive(c.keep_alive)
+                        .client_timeout(c.client_timeout)
+                        .disconnect_timeout(c.client_disconnect)
+                        .finish(map_config(factory(), move |_| cfg.clone()))
+                })?;
         Ok(self)
     }
 
@@ -303,26 +277,25 @@ where
         let cfg = self.config.clone();
         let addr = lst.local_addr().unwrap();
 
-        self.builder = self.builder.listen(
-            format!("ntex-web-service-{}", addr),
-            lst,
-            move || {
-                let c = cfg.lock().unwrap();
-                let cfg = AppConfig::new(
-                    true,
-                    addr,
-                    c.host.clone().unwrap_or_else(|| format!("{}", addr)),
-                );
-                HttpService::build()
-                    .keep_alive(c.keep_alive)
-                    .client_timeout(c.client_timeout)
-                    .disconnect_timeout(c.client_disconnect)
-                    .ssl_handshake_timeout(c.handshake_timeout)
-                    .memory_pool(c.pool)
-                    .finish(map_config(factory(), move |_| cfg.clone()))
-                    .openssl(acceptor.clone())
-            },
-        )?;
+        self.builder =
+            self.builder
+                .listen(format!("ntex-web-service-{}", addr), lst, move |r| {
+                    let c = cfg.lock().unwrap();
+                    let cfg = AppConfig::new(
+                        true,
+                        addr,
+                        c.host.clone().unwrap_or_else(|| format!("{}", addr)),
+                    );
+                    r.memory_pool(c.pool);
+
+                    HttpService::build()
+                        .keep_alive(c.keep_alive)
+                        .client_timeout(c.client_timeout)
+                        .disconnect_timeout(c.client_disconnect)
+                        .ssl_handshake_timeout(c.handshake_timeout)
+                        .finish(map_config(factory(), move |_| cfg.clone()))
+                        .openssl(acceptor.clone())
+                })?;
         Ok(self)
     }
 
@@ -351,19 +324,20 @@ where
         self.builder = self.builder.listen(
             format!("ntex-web-rustls-service-{}", addr),
             lst,
-            move || {
+            move |r| {
                 let c = cfg.lock().unwrap();
                 let cfg = AppConfig::new(
                     true,
                     addr,
                     c.host.clone().unwrap_or_else(|| format!("{}", addr)),
                 );
+                r.memory_pool(c.pool);
+
                 HttpService::build()
                     .keep_alive(c.keep_alive)
                     .client_timeout(c.client_timeout)
                     .disconnect_timeout(c.client_disconnect)
                     .ssl_handshake_timeout(c.handshake_timeout)
-                    .memory_pool(c.pool)
                     .finish(map_config(factory(), move |_| cfg.clone()))
                     .rustls(config.clone())
             },
@@ -384,10 +358,7 @@ where
         Ok(self)
     }
 
-    fn bind2<A: net::ToSocketAddrs>(
-        &self,
-        addr: A,
-    ) -> io::Result<Vec<net::TcpListener>> {
+    fn bind2<A: net::ToSocketAddrs>(&self, addr: A) -> io::Result<Vec<net::TcpListener>> {
         let mut err = None;
         let mut succ = false;
         let mut sockets = Vec::new();
@@ -457,38 +428,27 @@ where
     /// Start listening for unix domain connections on existing listener.
     ///
     /// This method is available with `uds` feature.
-    pub fn listen_uds(
-        mut self,
-        lst: std::os::unix::net::UnixListener,
-    ) -> io::Result<Self> {
-        use crate::rt::net::UnixStream;
-
+    pub fn listen_uds(mut self, lst: std::os::unix::net::UnixListener) -> io::Result<Self> {
         let cfg = self.config.clone();
         let factory = self.factory.clone();
-        let socket_addr = net::SocketAddr::new(
-            net::IpAddr::V4(net::Ipv4Addr::new(127, 0, 0, 1)),
-            8080,
-        );
+        let socket_addr =
+            net::SocketAddr::new(net::IpAddr::V4(net::Ipv4Addr::new(127, 0, 0, 1)), 8080);
 
         let addr = format!("ntex-web-service-{:?}", lst.local_addr()?);
 
-        self.builder = self.builder.listen_uds(addr, lst, move || {
+        self.builder = self.builder.listen_uds(addr, lst, move |r| {
             let c = cfg.lock().unwrap();
             let config = AppConfig::new(
                 false,
                 socket_addr,
                 c.host.clone().unwrap_or_else(|| format!("{}", socket_addr)),
             );
-            pipeline_factory(|io: UnixStream| {
-                crate::util::Ready::Ok((io, Protocol::Http1, None))
-            })
-            .and_then(
-                HttpService::build()
-                    .keep_alive(c.keep_alive)
-                    .client_timeout(c.client_timeout)
-                    .memory_pool(c.pool)
-                    .finish(map_config(factory(), move |_| config.clone())),
-            )
+            r.memory_pool(c.pool);
+
+            HttpService::build()
+                .keep_alive(c.keep_alive)
+                .client_timeout(c.client_timeout)
+                .finish(map_config(factory(), move |_| config.clone()))
         })?;
         Ok(self)
     }
@@ -501,35 +461,27 @@ where
     where
         A: AsRef<std::path::Path>,
     {
-        use crate::rt::net::UnixStream;
-
         let cfg = self.config.clone();
         let factory = self.factory.clone();
-        let socket_addr = net::SocketAddr::new(
-            net::IpAddr::V4(net::Ipv4Addr::new(127, 0, 0, 1)),
-            8080,
-        );
+        let socket_addr =
+            net::SocketAddr::new(net::IpAddr::V4(net::Ipv4Addr::new(127, 0, 0, 1)), 8080);
 
         self.builder = self.builder.bind_uds(
             format!("ntex-web-service-{:?}", addr.as_ref()),
             addr,
-            move || {
+            move |r| {
                 let c = cfg.lock().unwrap();
                 let config = AppConfig::new(
                     false,
                     socket_addr,
                     c.host.clone().unwrap_or_else(|| format!("{}", socket_addr)),
                 );
-                pipeline_factory(|io: UnixStream| {
-                    crate::util::Ready::Ok((io, Protocol::Http1, None))
-                })
-                .and_then(
-                    HttpService::build()
-                        .keep_alive(c.keep_alive)
-                        .client_timeout(c.client_timeout)
-                        .memory_pool(c.pool)
-                        .finish(map_config(factory(), move |_| config.clone())),
-                )
+                r.memory_pool(c.pool);
+
+                HttpService::build()
+                    .keep_alive(c.keep_alive)
+                    .client_timeout(c.client_timeout)
+                    .finish(map_config(factory(), move |_| config.clone()))
             },
         )?;
         Ok(self)
@@ -539,8 +491,8 @@ where
 impl<F, I, S, B> HttpServer<F, I, S, B>
 where
     F: Fn() -> I + Send + Clone + 'static,
-    I: IntoServiceFactory<S>,
-    S: ServiceFactory<Config = AppConfig, Request = Request>,
+    I: IntoServiceFactory<S, Request, AppConfig>,
+    S: ServiceFactory<Request, AppConfig>,
     S::Error: ResponseError,
     S::InitError: fmt::Debug,
     S::Response: Into<Response<B>>,
@@ -570,7 +522,7 @@ where
     /// }
     /// ```
     pub fn run(self) -> Server {
-        self.builder.start()
+        self.builder.run()
     }
 }
 
